// ======================================================================
// ePost.cpp - main program of an English Part-of-Speech Tagger
// 
// 102300: Benjamin Han <benhdj@cs.cmu.edu> Created.
// 110500: Benjamin Han <benhdj@cs.cmu.edu> Added a simple exception for
//         rudimentary file IO error handling.
// 080401: Benjamin Han <benhdj@cs.cmu.edu> Adapted for MSVC++; chaned 
//         license to LGPL.
// 071802: Benjamin Han <benhdj@cs.cmu.edu> Added #include <iterator> for
//         gcc 3.1.
// ======================================================================

//   Copyright (C) 2001 Benjamin Han <benhdj@cs.cmu.edu>
//   The original Transformation-based Part-of-Speech Tagger is written
//   by Eric Brill <brill@cs.jhu.edu> - see the separate copyright notice
//   under post directory
//
//    This library is free software; you can redistribute it and/or
//    modify it under the terms of the GNU Lesser General Public
//    License as published by the Free Software Foundation; either
//    version 2.1 of the License, or (at your option) any later version.
//
//    This library is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//    Lesser General Public License for more details.
//
//    You should have received a copy of the GNU Lesser General Public
//    License along with this library; if not, write to the Free Software
//    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

#include "post.h"
#include "except.h"

#include <iterator>
#include <string>
#include <iostream>
#include <list>

#define BUF_SIZE 10240

using namespace std;

#define ERR_SUCCESS                    0
#define ERR_ILLEGAL_OPT                1
#define ERR_OPEN_INPUT_FILE            2
#define ERR_ILLEGAL_CORPUS_FILE_FORMAT 3
#define ERR_POSTERROR                  4

void printHelp ()
{
  cerr<<"* epost input-file"<<endl<<endl;
  cerr<<"  input-file: A formatted and tokenized English corpus."<<endl<<endl;  
  cerr<<"  *IMPORTANT*: DO NOT INCLUDE EMPTY LINES OR LINES WITH ONLY WHITE CHARS!"<<endl<<endl;
}

int main(int argc, char* argv[])
{

  FILE *fp;
  char buf[BUF_SIZE];

  string s;

  if (argc!=1)
    {
      printHelp();
      cerr<<"* Illegal command-line syntax."<<endl;
      return ERR_ILLEGAL_OPT;
    }

  POST post;

  while (fgets(buf,BUF_SIZE,stdin))
  {
    //printf("Readline: %s", buf);
    s=buf;
    if (s[s.length()-1]='\n') s.erase(s.length()-1);

    list<string> sList;
    sList.push_back(s);

    try
    {
      post.run(sList);
      copy(sList.begin(),sList.end(),ostream_iterator<string>(cout,"\n"));

      // gotta flush the file handle! otherwise, open2 in Perl causes
      // a nasty block
      fflush(stdout);
    }
    catch (Exception &e)
    {
      cerr<<"* ERROR: "<<e<<endl;
      return ERR_POSTERROR;
    }

  }
 
  return ERR_SUCCESS;
}
