#include <stdio.h>
#include <stdlib.h>

#include "lex.h"
#include "darray.h"
#include "useful.h"
#include "memory.h"
#include "tagger.h"

#define BUF_SIZE 10240

int main(int argc, char **argv) {
  Registry lexicon_hash, good_right_hash, good_left_hash;
  Registry seenTagging_hash;

  Darray lRuleArray,cRuleArray,bigramArray;

  FILE *lexicon, *bigrams, *lRuleFile, *cRuleFile;
  int i=0;

  int numSen = 1;
  char buf[BUF_SIZE];
  char **buffer = (char **) malloc(sizeof(char *));

  lexicon = fopen(LEXICON, "r");
  bigrams = fopen(BIGRAMS, "r");
  lRuleFile = fopen(LRULEFILE, "r");
  cRuleFile = fopen(CRULEFILE, "r");

  Tagger (lexicon, bigrams, lRuleFile, cRuleFile,
	  &lexicon_hash, &good_right_hash,
	  &good_left_hash, &seenTagging_hash,
	  &bigramArray, &lRuleArray, &cRuleArray);

  fclose(cRuleFile);
  fclose(lRuleFile);
  fclose(bigrams);
  fclose(lexicon);

  while ( fgets(buf,BUF_SIZE/3,stdin) ) {
    buf[strlen(buf)-1] = '\0';
    fflush(stdin);

    buffer[0] = (char *) malloc((strlen(buf)+1)*sizeof(char *));
    strcpy(buffer[0], buf);

    // ================= Call the tagger now ================= 
    buffer = StartStateTagger (buffer, numSen, lexicon_hash,
			       good_right_hash, good_left_hash,
			       bigramArray, lRuleArray);

    // lexicon_hash is effectively the same as a word_hash, which
    // is read from the lexicon file and originally used here. We
    // just replace word_hash with lexicon_hash here (save memory!)
    buffer = FinalStateTagger (buffer, numSen, seenTagging_hash,
			       lexicon_hash, cRuleArray);
  
    // ============== End of call to the tagger ==============

    printf("%s\n", buffer[0]);
  }
  
  Registry_traverse(lexicon_hash,FreeRegistryEntry,NULL);
  Registry_destroy(lexicon_hash);

  Registry_traverse(good_right_hash,FreeRegistryEntry,NULL);
  Registry_destroy(good_right_hash);

  Registry_traverse(good_left_hash,FreeRegistryEntry,NULL);
  Registry_destroy(good_left_hash);

  Registry_traverse(seenTagging_hash,FreeRegistryEntry,NULL);
  Registry_destroy(seenTagging_hash);

  for (i=0;i<Darray_len(lRuleArray);++i)
    free((char*)Darray_get(lRuleArray,i));

  for (i=0;i<Darray_len(cRuleArray);++i)
    free((char*)Darray_get(cRuleArray,i));

  for (i=0;i<Darray_len(bigramArray);++i)
    free((char*)Darray_get(bigramArray,i));

  Darray_destroy(lRuleArray);
  Darray_destroy(cRuleArray);
  Darray_destroy(bigramArray);
}
