/*
 Java version of Brill's Part-of-Speech Tagger
 Copyright (C) 2003-2004, Jimmy Lin

 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, please visit 
 http://www.gnu.org/copyleft/gpl.html

 */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>

#include "jni.h"

#include "lex.h"
#include "darray.h"
#include "useful.h"
#include "memory.h"
#include "tagger.h"

#include "lexicon.h"

#define BUF_SIZE 10240

Registry lexicon_hash, good_right_hash, good_left_hash;
Registry seenTagging_hash;
Darray lRuleArray,cRuleArray,bigramArray;

#ifndef __WIN32__
/* Don't understand... for some reason, fmemopen prototype isn't defined. */
FILE *fmemopen(char *, int, char *);
#endif

char *tag(const char *buf);

JNIEXPORT void JNICALL
Java_edu_mit_csail_brill_BTagger_setup (JNIEnv * env, 
				     jobject thisObj) {
  FILE *lexicon, *bigrams, *lRuleFile, *cRuleFile;
  int i=0;
  int numSen = 1;
  char buf[BUF_SIZE];

#ifdef __WIN32__
  lexicon = fopen("LEXICON.dump", "w");
  fwrite(LEXICON, sizeof(char), strlen(LEXICON), lexicon);
  fclose(lexicon);
  lexicon = fopen("LEXICON.dump", "r");
  
  bigrams = fopen("BIGRAMS.dump", "w");
  fwrite(BIGRAMS, sizeof(char), strlen(BIGRAMS), bigrams);
  fclose(bigrams);
  bigrams = fopen("BIGRAMS.dump", "r");
  
  lRuleFile = fopen("LRULEFILE.dump", "w");
  fwrite(LRULEFILE, sizeof(char), strlen(LRULEFILE), lRuleFile);
  fclose(lRuleFile);
  lRuleFile = fopen("LRULEFILE.dump", "r");

  cRuleFile = fopen("CRULEFILE.dump", "w");
  fwrite(CRULEFILE, sizeof(char), strlen(CRULEFILE), cRuleFile);
  fclose(cRuleFile);
  cRuleFile = fopen("CRULEFILE.dump", "r");
#else
  lexicon = (FILE *) fmemopen(LEXICON, strlen(LEXICON), "r");
  bigrams = (FILE *) fmemopen(BIGRAMS, strlen(BIGRAMS), "r");
  lRuleFile = (FILE *) fmemopen(LRULEFILE, strlen(LRULEFILE), "r");
  cRuleFile = (FILE *) fmemopen(CRULEFILE, strlen(CRULEFILE), "r");
#endif

  Tagger (lexicon, bigrams, lRuleFile, cRuleFile,
	  &lexicon_hash, &good_right_hash,
	  &good_left_hash, &seenTagging_hash,
	  &bigramArray, &lRuleArray, &cRuleArray);

  fclose(cRuleFile);
  fclose(lRuleFile);
  fclose(bigrams);
  fclose(lexicon);

}


JNIEXPORT jstring JNICALL
Java_edu_mit_csail_brill_BTagger_tag (JNIEnv * env, 
				   jobject thisObj, 
				   jstring strMessage) {
  char *s;
  jclass string_class;
  jstring utf_str;
  const char *str = (*env)->GetStringUTFChars(env, strMessage, 0);

  s = tag(str);

  (*env)->ReleaseStringUTFChars(env, strMessage, str);

  string_class = (*env)->FindClass(env, "java/lang/String");
  utf_str = (*env)->NewStringUTF(env, s);

  free(s);
  return utf_str;
}


char *tag(const char *buf) {
  char **buffer = (char **) malloc(sizeof(char *));

  buffer[0] = (char *) malloc((strlen(buf)+1)*sizeof(char *));
  strcpy(buffer[0], buf);

  buffer = StartStateTagger (buffer, 1, lexicon_hash,
			     good_right_hash, good_left_hash,
			     bigramArray, lRuleArray);

  buffer = FinalStateTagger (buffer, 1, seenTagging_hash,
			     lexicon_hash, cRuleArray);
  
  return buffer[0];
}


JNIEXPORT void JNICALL
Java_edu_mit_csail_brill_BTagger_destroy (JNIEnv * env, 
				       jobject thisObj) {
  int i;

  Registry_traverse(lexicon_hash,FreeRegistryEntry,NULL);
  Registry_destroy(lexicon_hash);

  Registry_traverse(good_right_hash,FreeRegistryEntry,NULL);
  Registry_destroy(good_right_hash);

  Registry_traverse(good_left_hash,FreeRegistryEntry,NULL);
  Registry_destroy(good_left_hash);

  Registry_traverse(seenTagging_hash,FreeRegistryEntry,NULL);
  Registry_destroy(seenTagging_hash);

  for (i=0;i<Darray_len(lRuleArray);++i)
    free((char*)Darray_get(lRuleArray,i));

  for (i=0;i<Darray_len(cRuleArray);++i)
    free((char*)Darray_get(cRuleArray,i));

  for (i=0;i<Darray_len(bigramArray);++i)
    free((char*)Darray_get(bigramArray,i));

  Darray_destroy(lRuleArray);
  Darray_destroy(cRuleArray);
  Darray_destroy(bigramArray);
  
#ifdef __WIN32__
  unlink("LEXICON.dump");
  unlink("BIGRAMS.dump");
  unlink("LRULEFILE.dump");
  unlink("CRULEFILE.dump");
#endif
}
